#!/usr/bin/perl -w

package Stats::Util;
use Exporter 'import';           
@EXPORT_OK = qw(scrape_data get_stats commify);     

use strict;
use warnings;

use Statistics::Descriptive;
use LWP::UserAgent;
use HTML::TableExtract;

my $datafile = 'rawData.dat';
my %majors_hash;

## -- Data retrieval -- ##
sub retrieve_and_persist_data {
    my $url = 'http://localhost/data.html';    ## reset as needed
    my $ua = LWP::UserAgent->new;
    my $response = $ua->get($url);
    my $page;

    if ($response->is_success) {
	$page = $response->decoded_content; 
	extract_from_html($page);
    }
    else {
	die $response->status_line;
    }
}

sub extract_from_html {
    my ($page) = @_;
 
    my $row_headers = row_headers();
    my $col_headers = col_headers();
    # Extract the study areas/income table, storing each row as the
    # value of the first column. In the created hash, the first 
    # column (e.g., Engineering) is the key and other row entries
    # (e.g,. base income, percentiles) are the key's value.
    my $te = HTML::TableExtract->new(headers => $col_headers);
    $te->parse($page);

    my $i = 0;
    foreach my $row ($te->rows) {
	my @list = ();
	foreach my $item (@$row) {
	    if (defined($item)) {
		$item = strip($item);
		push(@list, $item) if length($item) > 0;
	    }
	}
	shift @list;  ## 1st element is the study area name, have this as a key
	$majors_hash{$row_headers->[$i]} = \@list;
	$i++;
    }
    write_data();  ## persist the extracted data
}

# Each row of data is formatted as follows:
#
#   Symbolic_ID num1 num2...numN
#
# The symbolic id is an area such as Engineering and
# the other fields consist of income data for this area.
sub write_data {
    open(OUTFILE, ">$datafile");
    my ($key, $value, @list);
    while (($key, $value) = each %majors_hash) {
	@list = ();
	foreach my $item (@$value) {
	    push @list, $item;
	}
	my $record = $key . ' ' . join(' ', @list);  # one record in file
	print OUTFILE $record, "\n";  # add a newline to terminate record
    }
    close(OUTFILE);
}

sub strip {
    my ($string) = @_;
    $string =~ s/^\s+//; # remove leading spaces
    $string =~ s/\s+$//; # remove trailing spaces
    $string;
}

## -- Descriptive statistics -- ##

# Return the basic data about majors and incomes.
sub scrape_data {
    retrieve_and_persist_data();
    return \%majors_hash;
}

# Read rows from the majors_hash, keyed on majors. The rows
# are the average salaries, with each column a year, for areas of study.
sub get_stats {
    my (@medians, @list);
    my $stats = Statistics::Descriptive::Full->new();
    my ($k, $v);
    while (($k, $v) = each %majors_hash) {
	my @values = @$v;
	push @medians, $values[0];
    }
    $stats->add_data(@medians);
    $stats->sort_data();
    push @list, '$' . commify(sprintf("%.2f", $stats->min()));
    push @list, '$' . commify(sprintf("%.2f", $stats->max()));
    push @list, '$' . commify(sprintf("%.2f", $stats->sample_range()));
    push @list, '$' . commify(sprintf("%.2f", $stats->mean()));
    push @list, '$' . commify(sprintf("%.2f", $stats->geometric_mean()));

    return \@list;
}

sub commify {
    my $num = reverse $_[0];
    $num =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
    return scalar reverse $num;
}

sub col_headers {
    return ["Area", 
	    "MedianWage", 
	    "25th-ptile", 
	    "75th-ptile", 
	    "Popularity",
	    "GradDegreePercent",
	    "BoostFromGradDegree"];
}

sub row_headers {
    return ["Engineering",
	    "Computing",
	    "Business",
	    "Health",
	    "Science",
	    "SocialScience",
	    "Agriculture",
	    "Journalism",
	    "IndustrialArts",
	    "PublicPolicy",
	    "ForeignLanguage",
	    "LiberalArts",
	    "Arts",
	    "Education",
	    "SocialWork"];
}

1;
